@@ -72,6 +72,7 @@
  ****************************************************************************/
 struct hmi_controller_layer {
 	struct ivi_layout_layer *ivilayer;
+	struct weston_output *output;
 	uint32_t id_layer;
 	int32_t x;
 	int32_t y;
@@ -95,6 +96,7 @@ struct hmi_server_setting {
 	uint32_t    application_layer_id;
 	uint32_t    workspace_background_layer_id;
 	uint32_t    workspace_layer_id;
+	uint32_t    input_panel_layer_id;
 	uint32_t    base_layer_id_offset;
 	int32_t     panel_height;
 	uint32_t    transition_duration;
@@ -118,6 +120,8 @@ struct hmi_controller {
 	/* List of struct hmi_controller_layer */
 	struct wl_list                      base_layer_list;
 	struct wl_list                      application_layer_list;
+	struct wl_list                      input_panel_layer_list;
+	struct hmi_controller_layer        *active_input_panel_layer;
 	struct hmi_controller_layer         workspace_background_layer;
 	struct hmi_controller_layer         workspace_layer;
 	enum ivi_hmi_controller_layout_mode layout_mode;
@@ -135,6 +139,11 @@ struct hmi_controller {
 	struct wl_listener                  surface_configured;
 	struct wl_listener                  desktop_surface_configured;
 
+	struct wl_listener                  input_panel_configured;
+	struct wl_listener                  input_panel_show;
+	struct wl_listener                  input_panel_hide;
+	struct wl_listener                  input_panel_update;
+
 	struct wl_client                   *user_interface;
 	struct ui_setting                   ui_setting;
 
@@ -153,13 +162,6 @@ struct launcher_info {
 /*****************************************************************************
  *  local functions
  ****************************************************************************/
-static void *
-mem_alloc(size_t size, char *file, int32_t line)
-{
-	return fail_on_null(calloc(1, size), size, file, line);
-}
-
-#define MEM_ALLOC(s) mem_alloc((s),__FILE__,__LINE__)
 
 static int32_t
 is_surf_in_ui_widget(struct hmi_controller *hmi_ctrl,
@@ -173,6 +175,11 @@ is_surf_in_ui_widget(struct hmi_controller *hmi_ctrl,
 			return 1;
 	}
 
+	const struct ivi_layout_surface_properties *props;
+	props = hmi_ctrl->interface->get_properties_of_surface(ivisurf);
+	if (props->surface_type == IVI_LAYOUT_SURFACE_TYPE_INPUT_PANEL)
+		return 1;
+
 	return 0;
 }
 
@@ -222,8 +229,8 @@ mode_divided_into_tiling(struct hmi_controller *hmi_ctrl,
 	int32_t surf_num = 0;
 	int32_t idx = 0;
 
-	surfaces = MEM_ALLOC(sizeof(*surfaces) * surface_length);
-	new_order = MEM_ALLOC(sizeof(*surfaces) * surface_length);
+	surfaces = xcalloc(surface_length, sizeof(*surfaces));
+	new_order = xcalloc(surface_length, sizeof(*surfaces));
 
 	for (i = 0; i < surface_length; i++) {
 		ivisurf = pp_surface[i];
@@ -297,8 +304,8 @@ mode_divided_into_sidebyside(struct hmi_controller *hmi_ctrl,
 	int32_t surf_num = 0;
 	int32_t idx = 0;
 
-	surfaces = MEM_ALLOC(sizeof(*surfaces) * surface_length);
-	new_order = MEM_ALLOC(sizeof(*surfaces) * surface_length);
+	surfaces = xcalloc(surface_length, sizeof(*surfaces));
+	new_order = xcalloc(surface_length, sizeof(*surfaces));
 
 	for (i = 0; i < surface_length; i++) {
 		ivisurf = pp_surface[i];
@@ -362,7 +369,7 @@ mode_fullscreen_someone(struct hmi_controller *hmi_ctrl,
 	int32_t surf_num = 0;
 	struct ivi_layout_surface **surfaces;
 
-	surfaces = MEM_ALLOC(sizeof(*surfaces) * surface_length);
+	surfaces = xcalloc(surface_length, sizeof(*surfaces));
 
 	for (i = 0; i < surface_length; i++) {
 		ivisurf = pp_surface[i];
@@ -412,7 +419,7 @@ mode_random_replace(struct hmi_controller *hmi_ctrl,
 	int32_t i = 0;
 	int32_t layer_idx = 0;
 
-	layers = MEM_ALLOC(sizeof(*layers) * hmi_ctrl->screen_num);
+	layers = xcalloc(hmi_ctrl->screen_num, sizeof(*layers));
 
 	wl_list_for_each(application_layer, layer_list, link) {
 		layers[layer_idx] = application_layer;
@@ -445,8 +452,6 @@ mode_random_replace(struct hmi_controller *hmi_ctrl,
 							     surface_y,
 							     surface_width,
 							     surface_height);
-
-		hmi_ctrl->interface->layer_add_surface(layers[layer_idx]->ivilayer, ivisurf);
 	}
 
 	free(layers);
@@ -484,15 +489,13 @@ switch_mode(struct hmi_controller *hmi_ctrl,
 	struct wl_list *layer = &hmi_ctrl->application_layer_list;
 	struct ivi_layout_surface **pp_surface = NULL;
 	int32_t surface_length = 0;
-	int32_t ret = 0;
 
 	if (!hmi_ctrl->is_initialized)
 		return;
 
 	hmi_ctrl->layout_mode = layout_mode;
 
-	ret = hmi_ctrl->interface->get_surfaces(&surface_length, &pp_surface);
-	assert(!ret);
+	hmi_ctrl->interface->get_surfaces(&surface_length, &pp_surface);
 
 	if (!has_application_surface(hmi_ctrl, pp_surface, surface_length)) {
 		free(pp_surface);
@@ -554,25 +557,21 @@ create_layer(struct weston_output *output,
 	     struct hmi_controller_layer *layer,
 	     struct hmi_controller *hmi_ctrl)
 {
-	int32_t ret = 0;
-
 	layer->ivilayer =
 		hmi_ctrl->interface->layer_create_with_dimension(layer->id_layer,
 						       layer->width,
 						       layer->height);
 	assert(layer->ivilayer != NULL);
+	layer->output = output;
 
-	ret = hmi_ctrl->interface->screen_add_layer(output, layer->ivilayer);
-	assert(!ret);
+	hmi_ctrl->interface->screen_add_layer(output, layer->ivilayer);
 
-	ret = hmi_ctrl->interface->layer_set_destination_rectangle(layer->ivilayer,
+	hmi_ctrl->interface->layer_set_destination_rectangle(layer->ivilayer,
 							 layer->x, layer->y,
 							 layer->width,
 							 layer->height);
-	assert(!ret);
 
-	ret = hmi_ctrl->interface->layer_set_visibility(layer->ivilayer, true);
-	assert(!ret);
+	hmi_ctrl->interface->layer_set_visibility(layer->ivilayer, true);
 }
 
 /**
@@ -584,9 +583,25 @@ set_notification_remove_surface(struct wl_listener *listener, void *data)
 	struct hmi_controller *hmi_ctrl =
 			wl_container_of(listener, hmi_ctrl,
 					surface_removed);
-	(void)data;
+	struct ivi_layout_surface *ivisurf = data;
 
 	switch_mode(hmi_ctrl, hmi_ctrl->layout_mode);
+
+	/* set focus */
+	if (hmi_ctrl->interface->surface_is_active(ivisurf)) {
+		struct ivi_layout_surface **pp_surface = NULL;
+		int32_t surface_length = 0;
+		int32_t i;
+
+		hmi_ctrl->interface->get_surfaces(&surface_length, &pp_surface);
+
+		for (i = 0; i < surface_length; i++) {
+			if (pp_surface[i] != ivisurf) {
+				hmi_ctrl->interface->surface_activate(pp_surface[i]);
+				break;
+			}
+		}
+	}
 }
 
 static void
@@ -614,6 +629,8 @@ set_notification_configure_surface(struct wl_listener *listener, void *data)
 	 */
 	surface = hmi_ctrl->interface->surface_get_weston_surface(ivisurf);
 	if (surface) {
+		if (!weston_surface_has_content(surface))
+			return;
 		hmi_ctrl->interface->surface_set_source_rectangle(
 			ivisurf, 0, 0, surface->width,
 			surface->height);
@@ -642,6 +659,9 @@ set_notification_configure_surface(struct wl_listener *listener, void *data)
 		ivisurfs = NULL;
 	}
 
+	hmi_ctrl->interface->layer_add_surface(application_layer, ivisurf);
+	hmi_ctrl->interface->surface_activate(ivisurf);
+
 	switch_mode(hmi_ctrl, hmi_ctrl->layout_mode);
 }
 
@@ -658,14 +678,12 @@ set_notification_configure_desktop_surface(struct wl_listener *listener, void *d
 							link);
 	struct ivi_layout_layer *application_layer = layer_link->ivilayer;
 	struct weston_surface *surface;
-	int32_t ret = 0;
 
 	/* skip ui widgets */
 	if (is_surf_in_ui_widget(hmi_ctrl, ivisurf))
 		return;
 
-	ret = hmi_ctrl->interface->layer_add_surface(application_layer, ivisurf);
-	assert(!ret);
+	hmi_ctrl->interface->layer_add_surface(application_layer, ivisurf);
 
 	/*
 	 * if application changes size of wl_buffer. The source rectangle shall be
@@ -673,14 +691,118 @@ set_notification_configure_desktop_surface(struct wl_listener *listener, void *d
 	 */
 	surface = hmi_ctrl->interface->surface_get_weston_surface(ivisurf);
 	if (surface) {
+		if (!weston_surface_has_content(surface))
+			return;
 		hmi_ctrl->interface->surface_set_source_rectangle(ivisurf, 0,
 				0, surface->width, surface->height);
 	}
 
+	hmi_ctrl->interface->surface_activate(ivisurf);
 	hmi_ctrl->interface->commit_changes();
+
 	switch_mode(hmi_ctrl, hmi_ctrl->layout_mode);
 }
 
+static void
+set_notification_configure_input_panel_surface(struct wl_listener *listener, void *data)
+{
+	struct hmi_controller *hmi_ctrl =
+			wl_container_of(listener, hmi_ctrl,
+					input_panel_configured);
+	struct ivi_layout_surface *ivisurf = data;
+	struct weston_surface *surface;
+	struct hmi_controller_layer *layer_link;
+
+	wl_list_for_each(layer_link, &hmi_ctrl->input_panel_layer_list, link) {
+		hmi_ctrl->interface->layer_add_surface(layer_link->ivilayer, ivisurf);
+	}
+
+	surface = hmi_ctrl->interface->surface_get_weston_surface(ivisurf);
+	if (surface)
+		hmi_ctrl->interface->surface_set_source_rectangle(ivisurf, 0,
+				0, surface->width, surface->height);
+}
+
+static void
+update_input_panel_position(struct hmi_controller *hmi_ctrl,
+			    struct ivi_layout_text_input_state *state)
+{
+	struct weston_surface *surface;
+	int32_t x, y;
+
+	surface = hmi_ctrl->interface->surface_get_weston_surface(state->input_panel);
+
+	if (state->overlay_panel) {
+		const struct ivi_layout_surface_properties *props;
+
+		props = hmi_ctrl->interface->get_properties_of_surface(state->surface);
+		x = props->dest_x + state->cursor_rectangle.x2;
+		y = props->dest_y + state->cursor_rectangle.y2;
+	}
+	else {
+		x = (hmi_ctrl->active_input_panel_layer->width - surface->width) / 2;
+		y = hmi_ctrl->active_input_panel_layer->height - surface->height;
+	}
+
+	hmi_ctrl->interface->surface_set_destination_rectangle(state->input_panel,
+							       x, y,
+							       surface->width,
+							       surface->height);
+	hmi_ctrl->interface->surface_set_visibility(state->input_panel, true);
+	hmi_ctrl->interface->commit_changes();
+}
+
+static void
+set_notification_show_input_panel(struct wl_listener *listener, void *data)
+{
+	struct hmi_controller *hmi_ctrl =
+			wl_container_of(listener, hmi_ctrl,
+					input_panel_show);
+	struct ivi_layout_text_input_state *state = data;
+	struct hmi_controller_layer *layer_link;
+	struct weston_surface *focus;
+
+	focus = hmi_ctrl->interface->surface_get_weston_surface(state->surface);
+
+	wl_list_for_each(layer_link, &hmi_ctrl->input_panel_layer_list, link) {
+		if (layer_link->output != focus->output)
+			continue;
+
+		hmi_ctrl->interface->layer_set_visibility(layer_link->ivilayer, true);
+		hmi_ctrl->active_input_panel_layer = layer_link;
+		update_input_panel_position(hmi_ctrl, state);
+	}
+}
+
+static void
+set_notification_hide_input_panel(struct wl_listener *listener, void *data)
+{
+	struct hmi_controller *hmi_ctrl =
+			wl_container_of(listener, hmi_ctrl,
+					input_panel_hide);
+
+	if (!hmi_ctrl->active_input_panel_layer)
+		return;
+
+	hmi_ctrl->interface->layer_set_visibility(hmi_ctrl->active_input_panel_layer->ivilayer, false);
+	hmi_ctrl->active_input_panel_layer = NULL;
+	hmi_ctrl->interface->commit_changes();
+}
+
+static void
+set_notification_update_input_panel(struct wl_listener *listener, void *data)
+{
+	struct hmi_controller *hmi_ctrl =
+			wl_container_of(listener, hmi_ctrl,
+					input_panel_update);
+	struct ivi_layout_text_input_state *state = data;
+
+	if (!hmi_ctrl->active_input_panel_layer)
+		return;
+
+	update_input_panel_position(hmi_ctrl, state);
+}
+
 /**
  * A hmi_controller used 4 ivi_layers to manage ivi_surfaces. The IDs of
  * corresponding ivi_layer are defined in weston.ini. Default scene graph
@@ -689,7 +811,7 @@ set_notification_configure_desktop_surface(struct wl_listener *listener, void *d
 static struct hmi_server_setting *
 hmi_server_setting_create(struct weston_compositor *ec)
 {
-	struct hmi_server_setting *setting = MEM_ALLOC(sizeof(*setting));
+	struct hmi_server_setting *setting = xzalloc(sizeof(*setting));
 	struct weston_config *config = wet_get_config(ec);
 	struct weston_config_section *shell_section = NULL;
 	char *ivi_ui_config;
@@ -711,6 +833,9 @@ hmi_server_setting_create(struct weston_compositor *ec)
 	weston_config_section_get_uint(shell_section, "application-layer-id",
 				       &setting->application_layer_id, 4000);
 
+	weston_config_section_get_uint(shell_section, "input-panel-layer-id",
+				       &setting->input_panel_layer_id, 5000);
+
 	weston_config_section_get_uint(shell_section, "base-layer-id-offset",
 				       &setting->base_layer_id_offset, 10000);
 
@@ -745,6 +870,8 @@ hmi_controller_destroy(struct wl_listener *listener, void *data)
 	struct hmi_controller *hmi_ctrl =
 		container_of(listener, struct hmi_controller, destroy_listener);
 
+	wl_list_remove(&hmi_ctrl->destroy_listener.link);
+
 	wl_list_for_each_safe(link, next,
 			      &hmi_ctrl->workspace_fade.layer_list, link) {
 		wl_list_remove(&link->link);
@@ -765,6 +892,17 @@ hmi_controller_destroy(struct wl_listener *listener, void *data)
 		free(ctrl_layer_link);
 	}
 
+	/* clear input_panel_layer_list */
+	wl_list_for_each_safe(ctrl_layer_link, ctrl_layer_next,
+			      &hmi_ctrl->input_panel_layer_list, link) {
+		wl_list_remove(&ctrl_layer_link->link);
+		free(ctrl_layer_link);
+	}
+
+	wl_list_remove(&hmi_ctrl->surface_removed.link);
+	wl_list_remove(&hmi_ctrl->surface_configured.link);
+	wl_list_remove(&hmi_ctrl->desktop_surface_configured.link);
+
 	wl_array_release(&hmi_ctrl->ui_widgets);
 	free(hmi_ctrl->hmi_setting);
 	free(hmi_ctrl);
@@ -794,6 +932,7 @@ hmi_controller_create(struct weston_compositor *ec)
 	const struct ivi_layout_interface *interface;
 	struct hmi_controller_layer *base_layer = NULL;
 	struct hmi_controller_layer *application_layer = NULL;
+	struct hmi_controller_layer *input_panel_layer = NULL;
 	struct weston_output *output;
 	int32_t i;
 
@@ -804,7 +943,7 @@ hmi_controller_create(struct weston_compositor *ec)
 		return NULL;
 	}
 
-	hmi_ctrl = MEM_ALLOC(sizeof(*hmi_ctrl));
+	hmi_ctrl = xzalloc(sizeof(*hmi_ctrl));
 	i = 0;
 
 	wl_array_init(&hmi_ctrl->ui_widgets);
@@ -817,7 +956,7 @@ hmi_controller_create(struct weston_compositor *ec)
 	/* init base ivi_layer*/
 	wl_list_init(&hmi_ctrl->base_layer_list);
 	wl_list_for_each(output, &ec->output_list, link) {
-		base_layer = MEM_ALLOC(1 * sizeof(struct hmi_controller_layer));
+		base_layer = xzalloc(sizeof(struct hmi_controller_layer));
 		base_layer->x = 0;
 		base_layer->y = 0;
 		base_layer->width = output->current_mode->width;
@@ -837,7 +976,7 @@ hmi_controller_create(struct weston_compositor *ec)
 	/* init application ivi_layer */
 	wl_list_init(&hmi_ctrl->application_layer_list);
 	wl_list_for_each(output, &ec->output_list, link) {
-		application_layer = MEM_ALLOC(1 * sizeof(struct hmi_controller_layer));
+		application_layer = xzalloc(sizeof(struct hmi_controller_layer));
 		application_layer->x = 0;
 		application_layer->y = 0;
 		application_layer->width = output->current_mode->width;
@@ -870,9 +1009,27 @@ hmi_controller_create(struct weston_compositor *ec)
 	hmi_ctrl->interface->layer_set_visibility(
 		hmi_ctrl->workspace_background_layer.ivilayer, false);
 
+	i = 0;
+	/* init input panel ivi_layer */
+	wl_list_init(&hmi_ctrl->input_panel_layer_list);
+	wl_list_for_each(output, &ec->output_list, link) {
+		input_panel_layer = xzalloc(sizeof(struct hmi_controller_layer));
+		input_panel_layer->x = 0;
+		input_panel_layer->y = 0;
+		input_panel_layer->width = output->current_mode->width;
+		input_panel_layer->height = output->current_mode->height - panel_height;
+		input_panel_layer->id_layer =
+			hmi_ctrl->hmi_setting->input_panel_layer_id +
+						(i * hmi_ctrl->hmi_setting->base_layer_id_offset);
+		wl_list_insert(&hmi_ctrl->input_panel_layer_list, &input_panel_layer->link);
+
+		create_layer(output, input_panel_layer, hmi_ctrl);
+		hmi_ctrl->interface->layer_set_visibility(input_panel_layer->ivilayer, false);
+		i++;
+	}
 
 	wl_list_init(&hmi_ctrl->workspace_fade.layer_list);
-	tmp_link_layer = MEM_ALLOC(sizeof(*tmp_link_layer));
+	tmp_link_layer = xzalloc(sizeof(*tmp_link_layer));
 	tmp_link_layer->layout_layer =
 		hmi_ctrl->workspace_background_layer.ivilayer;
 	wl_list_insert(&hmi_ctrl->workspace_fade.layer_list,
@@ -887,9 +1044,24 @@ hmi_controller_create(struct weston_compositor *ec)
 	hmi_ctrl->desktop_surface_configured.notify = set_notification_configure_desktop_surface;
 	hmi_ctrl->interface->add_listener_configure_desktop_surface(&hmi_ctrl->desktop_surface_configured);
 
-	hmi_ctrl->destroy_listener.notify = hmi_controller_destroy;
-	wl_signal_add(&hmi_ctrl->compositor->destroy_signal,
-		      &hmi_ctrl->destroy_listener);
+	hmi_ctrl->input_panel_configured.notify = set_notification_configure_input_panel_surface;
+	hmi_ctrl->interface->add_listener_configure_input_panel_surface(&hmi_ctrl->input_panel_configured);
+
+	hmi_ctrl->input_panel_show.notify = set_notification_show_input_panel;
+	hmi_ctrl->interface->add_listener_show_input_panel(&hmi_ctrl->input_panel_show);
+
+	hmi_ctrl->input_panel_hide.notify = set_notification_hide_input_panel;
+	hmi_ctrl->interface->add_listener_hide_input_panel(&hmi_ctrl->input_panel_hide);
+
+	hmi_ctrl->input_panel_update.notify = set_notification_update_input_panel;
+	hmi_ctrl->interface->add_listener_update_input_panel(&hmi_ctrl->input_panel_update);
+
+	if(hmi_ctrl->interface->shell_add_destroy_listener_once(&hmi_ctrl->destroy_listener,
+				hmi_controller_destroy) == IVI_FAILED){
+		hmi_controller_destroy(&hmi_ctrl->destroy_listener, NULL);
+		return NULL;
+	}
+
 
 	return hmi_ctrl;
 }
@@ -916,7 +1088,6 @@ ivi_hmi_controller_set_background(struct hmi_controller *hmi_ctrl,
 	int32_t dsty;
 	int32_t width;
 	int32_t height;
-	int32_t ret = 0;
 	int32_t i = 0;
 
 	wl_list_for_each_reverse(base_layer, &hmi_ctrl->base_layer_list, link) {
@@ -932,15 +1103,12 @@ ivi_hmi_controller_set_background(struct hmi_controller *hmi_ctrl,
 		ivisurf = hmi_ctrl->interface->get_surface_from_id(*add_surface_id);
 		assert(ivisurf != NULL);
 
-		ret = hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
-		assert(!ret);
+		hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
 
-		ret = hmi_ctrl->interface->surface_set_destination_rectangle(ivisurf,
+		hmi_ctrl->interface->surface_set_destination_rectangle(ivisurf,
 						dstx, dsty, width, height);
-		assert(!ret);
 
-		ret = hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
-		assert(!ret);
+		hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
 
 		i++;
 	}
@@ -961,7 +1129,6 @@ ivi_hmi_controller_set_panel(struct hmi_controller *hmi_ctrl,
 	struct hmi_controller_layer *base_layer;
 	struct ivi_layout_layer   *ivilayer = NULL;
 	int32_t width;
-	int32_t ret = 0;
 	int32_t panel_height = hmi_ctrl->hmi_setting->panel_height;
 	const int32_t dstx = 0;
 	int32_t dsty = 0;
@@ -976,18 +1143,15 @@ ivi_hmi_controller_set_panel(struct hmi_controller *hmi_ctrl,
 		ivisurf = hmi_ctrl->interface->get_surface_from_id(*add_surface_id);
 		assert(ivisurf != NULL);
 
-		ret = hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
-		assert(!ret);
+		hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
 
 		dsty = base_layer->height - panel_height;
 		width = base_layer->width;
 
-		ret = hmi_ctrl->interface->surface_set_destination_rectangle(
+		hmi_ctrl->interface->surface_set_destination_rectangle(
 			ivisurf, dstx, dsty, width, panel_height);
-		assert(!ret);
 
-		ret = hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
-		assert(!ret);
+		hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
 
 		i++;
 	}
@@ -1014,7 +1178,6 @@ ivi_hmi_controller_set_button(struct hmi_controller *hmi_ctrl,
 	struct ivi_layout_layer   *ivilayer = base_layer->ivilayer;
 	const int32_t width  = 48;
 	const int32_t height = 48;
-	int32_t ret = 0;
 	int32_t panel_height = 0;
 	int32_t dstx = 0;
 	int32_t dsty = 0;
@@ -1025,20 +1188,17 @@ ivi_hmi_controller_set_button(struct hmi_controller *hmi_ctrl,
 	ivisurf = hmi_ctrl->interface->get_surface_from_id(id_surface);
 	assert(ivisurf != NULL);
 
-	ret = hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
-	assert(!ret);
+	hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
 
 	panel_height = hmi_ctrl->hmi_setting->panel_height;
 
 	dstx = (60 * number) + 15;
 	dsty = (base_layer->height - panel_height) + 5;
 
-	ret = hmi_ctrl->interface->surface_set_destination_rectangle(
+	hmi_ctrl->interface->surface_set_destination_rectangle(
 		ivisurf,dstx, dsty, width, height);
-	assert(!ret);
 
-	ret = hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
-	assert(!ret);
+	hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
 }
 
 /**
@@ -1058,7 +1218,6 @@ ivi_hmi_controller_set_home_button(struct hmi_controller *hmi_ctrl,
 							base_layer,
 							link);
 	struct ivi_layout_layer   *ivilayer = base_layer->ivilayer;
-	int32_t ret = 0;
 	int32_t size = 48;
 	int32_t panel_height = hmi_ctrl->hmi_setting->panel_height;
 	const int32_t dstx = (base_layer->width - size) / 2;
@@ -1071,15 +1230,12 @@ ivi_hmi_controller_set_home_button(struct hmi_controller *hmi_ctrl,
 	ivisurf = hmi_ctrl->interface->get_surface_from_id(id_surface);
 	assert(ivisurf != NULL);
 
-	ret = hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
-	assert(!ret);
+	hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
 
-	ret = hmi_ctrl->interface->surface_set_destination_rectangle(
+	hmi_ctrl->interface->surface_set_destination_rectangle(
 			ivisurf, dstx, dsty, size, size);
-	assert(!ret);
 
-	ret = hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
-	assert(!ret);
+	hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
 }
 
 /**
@@ -1097,7 +1253,6 @@ ivi_hmi_controller_set_workspacebackground(struct hmi_controller *hmi_ctrl,
 	struct ivi_layout_layer   *ivilayer = NULL;
 	const int32_t width  = hmi_ctrl->workspace_background_layer.width;
 	const int32_t height = hmi_ctrl->workspace_background_layer.height;
-	int32_t ret = 0;
 
 	uint32_t *add_surface_id = wl_array_add(&hmi_ctrl->ui_widgets,
 						sizeof(*add_surface_id));
@@ -1107,15 +1262,12 @@ ivi_hmi_controller_set_workspacebackground(struct hmi_controller *hmi_ctrl,
 	ivisurf = hmi_ctrl->interface->get_surface_from_id(id_surface);
 	assert(ivisurf != NULL);
 
-	ret = hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
-	assert(!ret);
+	hmi_ctrl->interface->layer_add_surface(ivilayer, ivisurf);
 
-	ret = hmi_ctrl->interface->surface_set_destination_rectangle(ivisurf,
+	hmi_ctrl->interface->surface_set_destination_rectangle(ivisurf,
 							   0, 0, width, height);
-	assert(!ret);
 
-	ret = hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
-	assert(!ret);
+	hmi_ctrl->interface->surface_set_visibility(ivisurf, true);
 }
 
 /**
@@ -1161,7 +1313,6 @@ ivi_hmi_controller_add_launchers(struct hmi_controller *hmi_ctrl,
 
 	int32_t x = 0;
 	int32_t y = 0;
-	int32_t ret = 0;
 	struct ivi_layout_surface* layout_surface = NULL;
 	uint32_t *add_surface_id = NULL;
 
@@ -1239,9 +1390,8 @@ ivi_hmi_controller_add_launchers(struct hmi_controller *hmi_ctrl,
 			hmi_ctrl->interface->get_surface_from_id(data->surface_id);
 		assert(layout_surface);
 
-		ret = hmi_ctrl->interface->surface_set_destination_rectangle(
+		hmi_ctrl->interface->surface_set_destination_rectangle(
 				layout_surface, x, y, icon_size, icon_size);
-		assert(!ret);
 
 		nx++;
 
@@ -1267,7 +1417,7 @@ ivi_hmi_controller_add_launchers(struct hmi_controller *hmi_ctrl,
 	hmi_ctrl->interface->layer_set_visibility(hmi_ctrl->workspace_layer.ivilayer,
 					false);
 
-	tmp_link_layer = MEM_ALLOC(sizeof(*tmp_link_layer));
+	tmp_link_layer = xzalloc(sizeof(*tmp_link_layer));
 	tmp_link_layer->layout_layer = hmi_ctrl->workspace_layer.ivilayer;
 	wl_list_insert(&hmi_ctrl->workspace_fade.layer_list,
 		       &tmp_link_layer->link);
@@ -1278,12 +1428,10 @@ ivi_hmi_controller_add_launchers(struct hmi_controller *hmi_ctrl,
 			hmi_ctrl->interface->get_surface_from_id(data->surface_id);
 		assert(layout_surface);
 
-		ret = hmi_ctrl->interface->layer_add_surface(hmi_ctrl->workspace_layer.ivilayer,
-								  layout_surface);
-		assert(!ret);
+		hmi_ctrl->interface->layer_add_surface(hmi_ctrl->workspace_layer.ivilayer,
+						       layout_surface);
 
-		ret = hmi_ctrl->interface->surface_set_visibility(layout_surface, true);
-		assert(!ret);
+		hmi_ctrl->interface->surface_set_visibility(layout_surface, true);
 	}
 
 	wl_array_release(&launchers);
@@ -1460,9 +1608,11 @@ pointer_move_workspace_grab_end(struct pointer_grab *grab)
 	struct pointer_move_grab *pnt_move_grab =
 		(struct pointer_move_grab *)grab;
 	struct ivi_layout_layer *layer = pnt_move_grab->base.layer;
+	wl_fixed_t x;
 
+	x = wl_fixed_from_double(grab->grab.pointer->grab_pos.c.x);
 	move_workspace_grab_end(&pnt_move_grab->move, grab->resource,
-				grab->grab.pointer->grab_x, layer);
+				x, layer);
 
 	weston_pointer_end_grab(grab->grab.pointer);
 }
@@ -1568,9 +1718,11 @@ pointer_move_grab_motion(struct weston_pointer_grab *grab,
 	struct hmi_controller *hmi_ctrl =
 			wl_resource_get_user_data(pnt_move_grab->base.resource);
 	wl_fixed_t pointer_pos[2];
+	struct weston_coord_global pos;
 
-	weston_pointer_motion_to_abs(grab->pointer, event,
-				     &pointer_pos[0], &pointer_pos[1]);
+	pos = weston_pointer_motion_to_abs(grab->pointer, event);
+	pointer_pos[0] = wl_fixed_from_double(pos.c.x);
+	pointer_pos[1] = wl_fixed_from_double(pos.c.y);
 	move_grab_update(&pnt_move_grab->move, pointer_pos);
 	layer_set_pos(hmi_ctrl, pnt_move_grab->base.layer,
 		      pnt_move_grab->move.pos[0], pnt_move_grab->move.pos[1]);
@@ -1756,11 +1908,13 @@ create_workspace_pointer_move(struct weston_pointer *pointer,
 			      struct wl_resource* resource)
 {
 	struct pointer_move_grab *pnt_move_grab =
-		MEM_ALLOC(sizeof(*pnt_move_grab));
+		xzalloc(sizeof(*pnt_move_grab));
 
 	pnt_move_grab->base.resource = resource;
-	move_grab_init_workspace(&pnt_move_grab->move, pointer->grab_x,
-				 pointer->grab_y, resource);
+	move_grab_init_workspace(&pnt_move_grab->move,
+				 wl_fixed_from_double(pointer->grab_pos.c.x),
+				 wl_fixed_from_double(pointer->grab_pos.c.y),
+				 resource);
 
 	return pnt_move_grab;
 }
@@ -1770,7 +1924,7 @@ create_workspace_touch_move(struct weston_touch *touch,
 			    struct wl_resource* resource)
 {
 	struct touch_move_grab *tch_move_grab =
-		MEM_ALLOC(sizeof(*tch_move_grab));
+		xzalloc(sizeof(*tch_move_grab));
 
 	tch_move_grab->base.resource = resource;
 	tch_move_grab->is_active = 1;
